<?php

namespace XeRoX;

use FML\Controls\Frame;
use FML\Controls\Gauge;
use FML\Controls\Label;
use FML\Controls\Labels\Label_Button;
use FML\Controls\Quad;
use FML\Controls\Quads\Quad_BgsPlayerCard;
use FML\ManiaLink;
use FML\Script\Features\Paging;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Commands\CommandListener;
use ManiaControl\ManiaControl;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Maps\Map;
use ManiaControl\Players\Player;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Utils\ColorUtil;
use Maniaplanet\DedicatedServer\Xmlrpc\FileException;
use MCTeam\KarmaPlugin;

/**
 * Created by PhpStorm.
 * User: XeRoX
 * Date: 24.06.2015
 * Time: 10:24
 */
class MapCleaner implements Plugin, CallbackListener, ManialinkPageAnswerListener, CommandListener {

	const SETTING_KARMA_DELETE    = 'Karma threshold for automatic delete';
	const SETTING_KARMA_ASK_ADMIN = 'Karma threshold to show in admin list';
	const SETTING_MIN_VOTES       = 'Minimum number of votes';
	const SETTING_MIN_TIME        = 'Minimum number of days before map can be deleted';
	const DEFAULT_KARMA_PLUGIN    = 'MCTeam\KarmaPlugin';
	const ACTION_REMOVE_MAP       = 'MapCleaner.RemoveMap';
	const ACTION_REMOVE_ALL_MAPS  = 'MapCleaner.RemoveAllMaps';

	/**@var ManiaControl $maniaControl */
	private $maniaControl;
	/**@var KarmaPlugin $karmaPlugin */
	private $karmaPlugin;
	/**@var Map $mapsAdmin */
	private $mapsAdmin;

	/**
	 * Prepare the Plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 */
	public static function prepare(\ManiaControl\ManiaControl $maniaControl) {
		// TODO: Implement prepare() method.
	}

	/**
	 * Load the plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 * @return bool
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;
		$this->karmaPlugin  = $this->maniaControl->getPluginManager()->getPlugin(self::DEFAULT_KARMA_PLUGIN);

		if (!$this->karmaPlugin) {
			$this->maniaControl->getChat()->sendError('Karma Plugin must be activated');
			return;
		}

		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_KARMA_DELETE, 0.1);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_KARMA_ASK_ADMIN, 0.2);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_MIN_VOTES, 10);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_MIN_TIME, 7);

		$this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_PLAYERMANIALINKPAGEANSWER, $this, 'handleManialinkPageAnswer');

		$this->maniaControl->getCommandManager()->registerCommandListener('karmaMaps', $this, 'command_karmaMaps', true, '');
		$this->maniaControl->getCommandManager()->registerCommandListener('CleanMaps', $this, 'command_checkMaps', true,'');
	}

	/**
	 * Unload the plugin and its Resources
	 */
	public function unload() {
		// TODO: Implement unload() method.
	}

	public function command_checkMaps($callback, Player $player){

		$this->checkMaps($player);
	}

	private function reloadWidget(Player $player){
		$this->maniaControl->getChat()->sendInformation('Reloading Widget', $player->login);
		$this->maniaControl->manialinkManager->closeWidget($player);
		$this->checkMaps($player);
	}

	private function checkMaps(Player $player) {

		$maps = $this->maniaControl->getMapManager()->getMaps();
		$this->maniaControl->getChat()->sendInformation('Checking Maps', $player->login);

		$this->mapsAdmin  = array();
		$this->mapsDelete = array();

		$karmaDelete = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_KARMA_DELETE);
		$karmaAdmin  = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_KARMA_ASK_ADMIN);
		$minVotes    = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MIN_VOTES);
		$timeUntilDeleteUnix = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MIN_TIME) * 24 * 60 * 60;
		$currentTime = time();

		$displayMxKarma = $this->maniaControl->getSettingManager()->getSettingValue($this->karmaPlugin, karmaPlugin::SETTING_WIDGET_DISPLAY_MX);

		$k = 0;

		foreach ($maps as $map) {

			//Display Mx Karma
			if ($displayMxKarma && $map->mx) {
				$karma = $map->mx->ratingVoteAverage / 100;
				$votes = array("count" => $map->mx->ratingVoteCount);

				//Display Local Karma
			} else {
				$karma = $this->karmaPlugin->getMapKarma($map);
				$votes = $this->karmaPlugin->getMapVotes($map);
			}

			$dateAdded = $map->lastUpdate;

			if (is_numeric($karma) && $votes['count'] > 0) {
				if ($this->maniaControl->getSettingManager()->getSettingValue($this->karmaPlugin, karmaPlugin::SETTING_NEWKARMA)
				) {
					intval($votes = $votes['count']);

				} else {
					$min  = 0;
					$plus = 0;
					foreach ($votes as $vote) {
						if (isset($vote->vote)) {
							if ($vote->vote !== 0.5) {
								if ($vote->vote < 0.5) {
									$min = $min + $vote->count;
								} else {
									$plus = $plus + $vote->count;
								}
							}
						}
					}
					$karma = $plus - $min;
					intval($votes = $votes['count']);
				}
			}
			$karma    = floatval($karma);
			$votes    = intval($votes);
			$minVotes = intval($minVotes);


			if ($karma <= $karmaDelete && $votes >= $minVotes && $currentTime - $dateAdded > $timeUntilDeleteUnix) {
				$this->removeMap($map->uid, $player);
				$k++;
			} elseif ($karma <= $karmaAdmin && $votes >= $minVotes && $currentTime - $dateAdded > $timeUntilDeleteUnix) {
				$map->karma        = $karma;
				$map->votes        = $votes;
				$this->mapsAdmin[] = $map;
			}
		}

		if($k == 1){
			$message = '$4b5' . $k . ' map successfully cleaned';
		}else if($k > 1){
			$message = '$4b5' . $k . ' maps successfully cleaned';
		}else{
			$message = 'No maps auto-deleted';
		}

		$this->maniaControl->getChat()->sendInformation($message, $player->login);
		$this->showMapCleanerList($player);
	}

	public function command_karmaMaps($callback, Player $player) {
		if($this->mapsAdmin){
			$this->showMapCleanerList($player);
		}else{
			$this->maniaControl->getChat()->sendError('Checking maps, please try again', $player->login);
			$this->checkMaps($player);
		}

	}

	private function showMapCleanerList(Player $player){

		$width  = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsWidth();
		$height = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsHeight();

		// create manialink
		$maniaLink = new ManiaLink(ManialinkManager::MAIN_MLID);
		$script    = $maniaLink->getScript();
		$paging    = new Paging();
		$script->addFeature($paging);

		// Main frame
		$frame = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultListFrame($script, $paging);
		$maniaLink->add($frame);

		// Start offsets
		$posX = -$width / 2;
		$posY = $height / 2;

		//Predefine description Label
		$descriptionLabel = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultDescriptionLabel();
		$frame->add($descriptionLabel);

		// Headline
		$headFrame = new Frame();
		$frame->add($headFrame);
		$headFrame->setY($posY - 5);

		$array = array('$oId' => $posX + 5, '$oMap' => $posX + 10, '$oAuthor' => $posX + 60, '$oKarma' => $posX + 90, '$oVotes' => $posX + 110, '$oDelete' => $posX + 130);

		$this->maniaControl->getManialinkManager()->labelLine($headFrame, $array);

		$index     = 1;
		$posY      = $posY - 10;
		$pageFrame = null;

		foreach ($this->mapsAdmin as $map) {

			if ($index % 15 === 1) {
				$pageFrame = new Frame();
				$frame->add($pageFrame);
				$posY = $height / 2 - 10;
				$paging->addPage($pageFrame);
			}

			$mapFrame = new Frame();
			$pageFrame->add($mapFrame);
			$mapFrame->setY($posY);

			if ($index % 2 !== 0) {
				$lineQuad = new Quad_BgsPlayerCard();
				$mapFrame->add($lineQuad);
				$lineQuad->setSize($width, 4);
				$lineQuad->setSubStyle($lineQuad::SUBSTYLE_BgPlayerCardBig);
				$lineQuad->setZ(0.001);
			}

			$texts     = array($index, $map->name, $map->authorNick, $map->votes);
			$positions = array($posX + 5, $posX + 10, $posX + 60, $posX + 110);


			$karmaGauge = new Gauge();
			$mapFrame->add($karmaGauge);
			$karmaGauge->setZ(2);
			$karmaGauge->setX($posX + 95);
			$karmaGauge->setSize(20, 9);
			$karmaGauge->setDrawBg(false);
			$karma = floatval($map->karma);
			$karmaGauge->setRatio($karma + 0.15 - $karma * 0.15);
			$karmaColor = ColorUtil::floatToStatusColor($karma);
			$karmaGauge->setColor($karmaColor . '9');

			$karmaLabel = new Label();
			$mapFrame->add($karmaLabel);
			$karmaLabel->setZ(2);
			$karmaLabel->setX($posX + 95);
			$karmaLabel->setSize(20 * 0.9, 5);
			$karmaLabel->setTextSize(0.9);
			$karmaLabel->setTextColor('fff');
			$karmaLabel->setText(round($map->karma,3));

			// remove map button
			$removeButton = new Label_Button();
			$mapFrame->add($removeButton);
			$removeButton->setX($posX + 135);
			$removeButton->setZ(0.2);
			$removeButton->setSize(20, 3);
			$removeButton->setTextSize(1);
			$removeButton->setText('Delete');
			$removeButton->setTextColor('f43');

			$confirmFrame = $this->buildConfirmFrame($maniaLink, $posY, $map->uid);
			$removeButton->addToggleFeature($confirmFrame);
			$description = 'Remove Map: ' . $map->getEscapedName();
			$removeButton->addTooltipLabelFeature($descriptionLabel, $description);

			$this->maniaControl->getManialinkManager()->labelLine($mapFrame, array($positions, $texts));

			$posY -= 4;
			$index++;

		}

		$label = new Label_Button();
		$frame->add($label);
		$label->setSize(20, 5);
		$label->setText('Delete all maps');
		$label->setTextColor('f43');
		$label->setTextSize('2');
		$label->setPosition(-$width / 2 + 11, -$height / 2 + 3);
		$label->setAction(self::ACTION_REMOVE_ALL_MAPS);

		$this->maniaControl->getManialinkManager()->sendManialink($maniaLink, $player);
	}

	public function handleManialinkPageAnswer(array $callback) {
		$actionId    = $callback[1][2];
		$actionArray = explode('.', $actionId, 3);

		if (count($actionArray) <= 2) {
			$action = $actionId;
		}else{
			$id     = $actionArray[2];
			$action = $actionArray[0] . '.' . $actionArray[1];
		}

		$login  = $callback[1][1];

		$player = $this->maniaControl->getPlayerManager()->getPlayer($login);

		switch ($action) {
			case(self::ACTION_REMOVE_MAP):
				$this->removeMap($id, $player);
				$this->reloadWidget($player);
				break;
			case(self::ACTION_REMOVE_ALL_MAPS):
				foreach ($this->mapsAdmin as $map) {
					$this->removeMap($map->uid, $player);
				}
				$this->reloadWidget($player);
		}

	}

	private function removeMap($id,Player $player) {
		try {
			if($this->maniaControl->getMapManager()->getMapByUid($id) != $this->maniaControl->getMapManager()->getCurrentMap()) {
				$this->maniaControl->getMapManager()->removeMap($player,$id);
			}else{
				$this->maniaControl->getChat()->sendError('You cannot delete the map you are playing on', $player);
			}
		} catch (FileException $e) {
			$this->maniaControl->getChat()->sendException($e, $player);
		}
	}

	/**
	 * @param \FML\ManiaLink $maniaLink
	 * @param                $posY
	 * @param                $mapUid
	 * @return \FML\Controls\Frame
	 */
	public function buildConfirmFrame(Manialink $maniaLink, $posY, $mapUid) {
		// TODO: get rid of the confirm frame to decrease xml size & network usage
		// SUGGESTION: just send them as own manialink again on clicking?

		$width        = $this->maniaControl->getManialinkManager()->getStyleManager()->getListWidgetsWidth();
		$quadStyle    = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultMainWindowStyle();
		$quadSubstyle = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultMainWindowSubStyle();

		$confirmFrame = new Frame();
		$maniaLink->add($confirmFrame);
		$confirmFrame->setPosition($width / 2 + 6, $posY);
		$confirmFrame->setVisible(false);

		$quad = new Quad();
		$confirmFrame->add($quad);
		$quad->setStyles($quadStyle, $quadSubstyle);
		$quad->setSize(12, 4);

		$quad = new Quad_BgsPlayerCard();
		$confirmFrame->add($quad);
		$quad->setSubStyle($quad::SUBSTYLE_BgCardSystem);
		$quad->setSize(11, 3.5);

		$label = new Label_Button();
		$confirmFrame->add($label);
		$label->setText('Sure?');
		$label->setTextSize(1);
		$label->setScale(0.90);
		$label->setX(-1.3);

		$buttLabel = new Label_Button();
		$confirmFrame->add($buttLabel);
		$buttLabel->setPosition(3.2, 0.4, 0.2);
		$buttLabel->setSize(3, 3);

		$buttLabel->setTextSize(1);
		$buttLabel->setTextColor('a00');
		$buttLabel->setText('x');
		$quad->setAction(self::ACTION_REMOVE_MAP . '.' . $mapUid);

		return $confirmFrame;
	}

	/**
	 * Get plugin id
	 *
	 * @return int
	 */
	public static function getId() {
		return 79;
	}

	/**
	 * Get Plugin Name
	 *
	 * @return string
	 */
	public static function getName() {
		return 'MapCleaner';
	}

	/**
	 * Get Plugin Version
	 *
	 * @return string
	 */
	public static function getVersion() {
		return 0.11;
	}

	/**
	 * Get Plugin Author
	 *
	 * @return string
	 */
	public static function getAuthor() {
		return 'XeRoX';
	}

	/**
	 * Get Plugin Description
	 *
	 * @return string
	 */
	public static function getDescription() {
		return 'none (yet)';
	}

}